const St = imports.gi.St;
const Clutter = imports.gi.Clutter;

const Lang = imports.lang;
const Main = imports.ui.main;
const DND = imports.ui.dnd;

var xdndButton;

/**
 * XDnD test class, wraps StButton.
 */
const XdndButton = new Lang.Class({
	Name: "XdndButton",

	/**
	 * Initialize class.
	 */
	_init: function() {

		/* Create button. */
		this.button = new St.Button({label: "[XDND]"});
		this.button.set_position(5, 30);
		this.button.set_background_color(new Clutter.Color({
			red: 0,
			blue: 0,
			green: 0,
			alpha: 150,
		}));

		/* Reset when clicked. */
		this.button.connect("clicked", Lang.bind(this, function() {
			this.button.set_label("[XDND]");
		}));

		/* Clean up when destroyed. */
		this.button.connect("destroy", Lang.bind(this, function() {
			this.destroy();
		}));

		/* Enable drag and drop. */
		this.button._delegate = this;
		this.button.set_reactive(true);

		/* Add button to layout manager. */
		Main.layoutManager.addChrome(this.button);
	},

	/**
	 * Called by XdndHandler when something is dragged over the button.
	 */
	handleDragOver: function(source, actor, x, y, time) {
		let text = "[DRAG " + time + "] (" + Math.round(x) + ", " + Math.round(y) + ")";

		/* We are only interested in XDnD. */
		if (source.mimeTypes) {
			text += ":\n"
			text += source.mimeTypes.join("\n");

			/* Accept drops only when text/uri-list information is available. */
			if (0 <= source.mimeTypes.indexOf("text/uri-list")) {
				this.button.set_label(text);

				/* Accept the drop. */
				return DND.DragMotionResult.COPY_DROP;
			}
		}

		this.button.set_label(text);

		/* We are not be interested, but perhaps someone else is. */
		return DND.DragMotionResult.CONTINUE;
	},

	/**
	 * Called by XdndHandler when something is dropped on the button.
	 */
	acceptDrop: function(source, actor, x, y, time) {
		let text = "[DROP " + time + "] (" + Math.round(x) + ", " + Math.round(y) + ")";

		/* We are only interested in XDnD. */
		if (source.mimeTypes) {
			text += ":\n"
			text += source.mimeTypes.join("\n");

			/* Can we request text/uri-list data? */
			if (0 <= source.mimeTypes.indexOf("text/uri-list") && source.requestData) {

				/* Request the text/uri-list data. */
				source.requestData(this, "text/uri-list");
			}
		}

		this.button.set_label(text);
	},

	/**
	 * Called by XdndHandler when requested data has been received.
	 */
	acceptData: function(source, actor, x, y, time, mimeType, data) {
		/* Did we get the requested data? */
		if (mimeType === "text/uri-list") {
			this.button.set_label("[DATA " + time + "]" + " " + mimeType + ":\n" + data.trim());
		}
	},

	/**
	 * Clean up.
	 */
	destroy: function() {
		this.button.destroy();
		this.button = null;
	},
});

/**
 * Initialize extension.
 */
function init() {

}

/**
 * Enable extension.
 */
function enable() {
	xdndButton = new XdndButton();
}

/**
 * Disable extension.
 */
function disable() {
	xdndButton.destroy();
	xdndButton = null;
}
